\
\ Code for the DCC booster.
\
\ $ProjectVersion: 0.44 $
\

\ Ports used:
\    b0 (in)    H bridge thermal sensor
\    b1 (out)   Presence detected
\    b2 (out)   Inversion active
\    b3 (out)   Disconnection
\    b4 (in)    DCC signal
\    b5 (in)    Inversion allowed (XXXXX to be confirmed)
\    b6 (in)    Loco detected
\    b7 (in)    Short-circuit detected
\    c1 (out)   H bridge BRAKE input
\    c2 (out)   H bridge PWM input

hex

variable inversion-mode      \ Is inversion enabled?

\ Output disabled is on portb3, H bridge control on portc1 and portc2

: enable-output ( -- )        \ PWM H, BRAKE L, DISCONN L
    1 portc bit-clear
    2 portc bit-set
    3 portb bit-clear
;

: disable-output ( -- )       \ PWM L, BRAKE H, DISCONN H
    1 portc bit-set
    2 portc bit-clear
    3 portb bit-set
 ;

\ Timer routines. At 10MhZ, with a prescaler of 2, each tick represent 0.8s.

: 100s ( -- )
    83 tmr0 !                     \ (0x100-0x83=0x7d, i.e. 0x7d*0.8s=100s)
    t0if intcon bit-clear         \ Clear overflow bit
    begin t0if intcon bit-set? /while repeat
;

: 300s ( -- ) 100s 100s 100s ;

\ Toggle inversion on portb2

: toggle-inversion ( -- ) 2 portb bit-toggle ;

: handle-shortcut ( -- )
    \ Toggle inversion if inversion is allowed
    inversion-mode @ if toggle-inversion then
    \ Wait for 100s
    100s
    \ If portb7 is now off, exit. Otherwise, disable the output and
    \ wait for 300s before reenabling it
    7 portb bit-set? /if exit then
    disable-output 300s enable-output
;

\ Maintain output disabled as long as thermal alert (portb0) is on

: handle-thermal-alert ( -- )
    disable-output
    begin 300s 0 portb bit-set? while repeat
    enable-output
;

\ If there is a shortcut on portb7, handle it

: mainloop ( -- )
    begin
	0 portb bit-set? if handle-thermal-alert then
	7 portb bit-set? if handle-shortcut then
    again ;

\ Initialization

: init ( -- )
    0 portb !                                \ Clear portb output latches
    f1 trisb !                               \ Port b 1, 2 and 3 as outputs
    5 portb bit-set? inversion-mode !        \ Inversion jumper is portb5
    0 portc !                                \ Clear portc output latches
    f9 trisc !                               \ Port c 1 and 2 as outputs
    0 option_reg !                           \ CLKOUT, prescaler(tmr0)=2
    enable-output
;

\ Main program
main
: main init mainloop ;

\ Configuration word

fosc-hs set-fosc    \ High-speed oscillator
